/* Set Reconciliation Implementation*/
// Include Files
#include "prob_recon_support.h"

// This function calculates Sample points and stores the result in a vector which is returned by the function.
// number_of - Number of desired sample points.
//fieldminus1 - for a field chosen to be over prime p, fieldminus1 = p-1.
vec_ZZ_p compute_samplepoints(long numberof,ZZ_p& fieldminus1)
{
  vec_ZZ_p temp;
  temp.SetLength(numberof);
  for(long ii=0;ii<numberof;ii++) //Setting samplepoints to 0,1,-1,2,-2 .....
    {
      if ((ii % 2)==0) temp[ii]=ii/2;
      else temp[ii]=fieldminus1-ii/2;
      if (ii  == 0) temp[ii]=0;
    }
  //  cout <<"\n"<<temp;
  return temp;
}


//  This function calculates the polynomial formed by the elements of "set" at the "sample points" and returns the vector containing the
// evaluated set polynomials at each sample point.
vec_ZZ_p calc_poly(vec_ZZ_p& set, vec_ZZ_p& samplepoints)
{
  long samplepointvectorsize=samplepoints.length();
  ZZ_p temp;
  vec_ZZ_p polyval;
  polyval.SetLength(samplepointvectorsize);
  for(long ii=0;ii<samplepointvectorsize;ii++)
    {
      conv(temp,to_ZZ("1"));
      for(long jj=0;jj<set.length();jj++)
      {
	temp*=(samplepoints[ii]-set[jj]);
      }
      polyval[ii]=temp;
    }

  return polyval;
}


//This function calculates "d" = |Sa|-|Sb|
long calc_diff_ma_mb(vec_ZZ_p A, vec_ZZ_p B)
{
  long temp = A.length()-B.length();
  return temp;
}


// This Function calculates ma_bar based on mbar and d
//It returns the floor value
long calc_mabar(long mbar,long d)
{
  long rtnvalue;
  if((mbar+d)%2!=0)
     rtnvalue=(mbar+d-1)/2;

  else
     rtnvalue=(mbar+d)/2;
  if (rtnvalue<=0) rtnvalue=1; //Added this in the part2 program to take care of parasitic case mbar=1.
   return rtnvalue;

}


// This Function calculates mb_bar based on mbar and d. Similar to the calc_mabar function.
long calc_mbbar(long mbar,long d)
{
   long rtnvalue;
   if((mbar-d)%2!=0)
      rtnvalue=(mbar-d-1)/2;
  else
      rtnvalue=(mbar-d)/2;
   if (rtnvalue<0) rtnvalue=0; //Added this in the part2 program to prevent negative problems
return rtnvalue;
}


// This function constructs the Vandemonde Matrix.
//RF is the Rational function vector (all the fi s) bmatrix is the RHS Matrix of size m bar.
mat_ZZ_p construct_vandermonde_matrix(const vec_ZZ_p& samplepoints,const vec_ZZ_p& RF, const vec_ZZ_p& bmatrix, const long mbar, const long d)
{
  //  cout <<"\nmbar = "<<mbar<<"\nd = "<<d;
  long mabar = calc_mabar(mbar,d);
  long mbbar=calc_mbbar(mbar,d);;
  //long mbbar=mbar-mabar;
  long powerterms;
  //cout <<"\nmabar = "<<mabar;
  //cout <<"\nmbbar = "<<mbbar;

  if(mabar > mbbar) powerterms=mabar;
  else powerterms = mbbar;

  vec_ZZ_p powersamplepoints;
  powersamplepoints.SetLength(powerterms);

  vec_vec_ZZ_p returnmatrix;
  returnmatrix.SetLength(mbar);

  for(long ii=0;ii<mbar;ii++)
    {
      returnmatrix[ii].SetLength(mabar+mbbar+1); //+1 is for the B matrix
           for(long pp=0;pp<powerterms;pp++) //Precalculating powers of sample point #ii
	  powersamplepoints[pp]=power(samplepoints[ii],pp);

      for(long jj=0;jj<mabar;jj++)
	returnmatrix[ii][jj]=powersamplepoints[mabar-jj-1];
      for(jj=0;jj<mbbar;jj++)
	returnmatrix[ii][jj+mabar]= -RF[ii]*powersamplepoints[mbbar-1-jj];
      returnmatrix[ii][mabar+mbbar]=bmatrix[ii]; //Augumented matrix
    }


  //Adding alternate rows (starting from row 1 to mbar-1; leaving out row 0)
  for(ii=1;ii<mbar-1;ii=ii+2)
    for(int jj=0;jj<=mabar+mbbar;jj++)
      returnmatrix[ii][jj]+=returnmatrix[ii+1][jj];

  //Dividing each alternate row by 2
  for(ii=1;ii<mbar;ii=ii+2)
    for(int jj=0;jj<=mabar+mbbar;jj++)
      returnmatrix[ii][jj]/=2;

  //Subtracting alternate rows (Leaving Row 0 alone) row 2 = row 2 - row 1 etc
  for(ii=2;ii<mbar;ii=ii+2)
    for(int jj=0;jj<=mabar+mbbar;jj++)
      returnmatrix[ii][jj]-=returnmatrix[ii-1][jj];

 mat_ZZ_p mat_rtn;
  mat_rtn.SetDims(mbar,mabar+mbbar+1);//+1 is for the B matrix
 MakeMatrix(mat_rtn,returnmatrix);
  return mat_rtn;
}


//Constructs the rhs for the vandrmonde matrix - the bmatrix . RF is the Rational Function Matrix. Rank is the rank of the LHS Matrix
vec_ZZ_p construct_RHS_constant_vector(vec_ZZ_p RF,vec_ZZ_p samplepoints,long rank,long mbar, long d)
{
  vec_ZZ_p rtnvalue;
  long mabar=calc_mabar(mbar,d);
  long mbbar=calc_mbbar(mbar,d);

  //long mbbar=mbar-mabar;
  rtnvalue.SetLength(rank);
  for(long ii=0;ii<rank;ii++)
      rtnvalue[ii]=RF[ii]*power(samplepoints[ii],mbbar)-power(samplepoints[ii],mabar);
  return rtnvalue;
}


//Reducing the A matrix into rank*rank size. (This is done after Gauss elimination
mat_ZZ_p makeA(mat_ZZ_p vmat,long rank,long mbar)
{
  vec_vec_ZZ_p tempmat;
  mat_ZZ_p tmat;
  tempmat.SetLength(rank);
  for(long ii=0;ii<rank;ii++)
  {
     tempmat[ii].SetLength(rank);
     for(long jj=0;jj<rank;jj++)
     {
       tempmat[ii][jj]=vmat[ii][jj];
     }
  }

  tmat.SetDims(rank,rank);

  MakeMatrix(tmat,tempmat);
  return tmat;
}


//Reducing the B matrix into rank*1 size
vec_ZZ_p makeB(mat_ZZ_p vmatrix,long rank,long mbar,long d)
{
  long mabar=calc_mabar(mbar,d);
  long mbbar=calc_mbbar(mbar,d);
  //cout <<"\n MBAR = "<<mbar;
  vec_ZZ_p temp;
  temp.SetLength(rank);
  for (long ii=0;ii<rank;ii++)
     temp[ii]=vmatrix[ii][mabar+mbbar];
  return temp;
}

//Solve by back substitution - mat is the LHS rank*rank matrix. bvec is the RHS rank size matrix

vec_ZZ_p backsubstitutesolve(mat_ZZ_p mat, vec_ZZ_p bvec, long rank,long totalcoeffs)
{
  vec_ZZ_p rtntemp;

  rtntemp.SetLength(totalcoeffs);
  //rtntemp.SetLength(rank);
  for(long ii=rank-1;ii>=0;ii--)
    {
      rtntemp[ii]=bvec[ii]/mat[ii][ii];
      for(long jj=0;jj<rank;jj++)
	{
	  bvec[jj]=bvec[jj]-rtntemp[ii]*mat[jj][ii];
	}
    }
  for(ii=rank;ii<totalcoeffs;ii++) //Setting the other coeffs - from index mbar-rank upto mbar to 0
    rtntemp[ii]=0;
  return rtntemp;
}


/* This function returns 0 if more Sample Points are needed, else returns a 1 if no more sample points are needed */
int match_value(ZZ_pX PX,ZZ_pX QX,vec_ZZ_p PCEVALS,vec_ZZ_p EXTRASPS,vec_ZZ_p EXTRAEVALS)
{
  long num_of_points = EXTRASPS.length();
  int returnvalue=1;
  ZZ_p p,q;
  for(int ii=0;ii<num_of_points;ii++)
    {
      p = eval(PX, EXTRASPS[ii]);
      q = eval(QX, EXTRASPS[ii]);
      //cout <<q;
      if (q !=0)
	{
	  if(EXTRAEVALS[ii]==0)
	    {
	      returnvalue=0;
	      break;
	    }
	  if (p/q != PCEVALS[ii]/EXTRAEVALS[ii])
	    {
	      returnvalue=0;
	      //      cout << "\nBreak1";
	      break;
	    }
	  // cout << "\nNo break";
	}
        else
	  {
	    if (EXTRAEVALS[ii]!=0)
	      {
		returnvalue=0;
		break;
	      }
	  }
    }
  return returnvalue;
}

vec_ZZ_p reconstruct(vec_ZZ_p B,long mm)
{
	vec_ZZ_p newvector;
	cin >> newvector;
	return newvector;
}
