/* Set Reconciliation Implementation*/
// Include Files

#include "ProbSync.h"



/* FUNCTION: reconcileProb                                             *
 * PURPOSE: This function implements the ProbSync algorithm. It returns*
 * a 2d integer array, with column zero (ie. result[0][x]) holding the *
 * hash values not present on the remote machine, and column one (ie.  *
 * result[1][x]) holding the hash values not present on the local      *
 * machine. The length of each column is held in the 0'th index of that*
 * column. (ie. if result[1][0] = 5 then index result[1][1] to         *
 * result[1][6] contain valid hash values.                             *
 * DATE: 5/22/02                                                       *
 * CONSTANTS: none                                                     *
 * MODULES: ProbSync                                                    */
int reconcileProb(long mbar,long bsize,vec_ZZ_p HAData,vec_ZZ_p HBSamplevalues, vec_ZZ_p EXTRASPS,vec_ZZ_p EXTRAEVALS,ZZ tempzz, int ***missingdata)
{
  //clock_t timevar1,timevar2,timevar3,timevar4,timevar5,timevar6,timevar7,timevar8,timevar9;
  //timevar1=clock();
  //time_t timesec;
  //timesec=time(NULL);
  //ZZ prime[7]; //Holds the primes

   vec_ZZ_p HASamplevalues; //A @ sample values
   // vec_ZZ_p HBSamplevalues; //B @ sample values
   vec_ZZ_p RF; //Rational function at sample points fi s
   vec_ZZ_p samplepoints; //Sample points stored here
   mat_ZZ_p vmatrix;//This will hold the Vandermonde matrix
   vec_ZZ_p bvector; //RHS of Vandemonde equation
   vec_ZZ_p bredvector; //The reduced RHS of the Vandermonde matrix size = rank
   ZZ_p fieldminus1; // holds p-1 for GF(p)


   //Determination of Samplepoints
   conv(fieldminus1, tempzz-1); //Setting up the value of fieldminus1
   samplepoints.SetLength(mbar);// mbar sample points needed
   samplepoints=compute_samplepoints(mbar,fieldminus1); //Compute these samplepoints
   //cout<<"\nSample Points:"<<samplepoints; //Display the samplepoints


   //Calculation of Polynomials formed by Set A & Set B at these sample points
   HASamplevalues.SetLength(mbar); //Set A
   HBSamplevalues.SetLength(mbar); //Set B
   HASamplevalues=calc_poly(HAData,samplepoints); //Calculate the polynomial formed by set A at the sample points

   vec_ZZ_p PCEVALS;
   PCEVALS.SetLength(EXTRASPS.length());
   PCEVALS = calc_poly(HAData,EXTRASPS); //Extra sample points evaluation for probabilistic

// HBSamplevalues=calc_poly(HBData,samplepoints); //Calculate the polynomial formed by set B at the sample points
   //timevar2=clock();
   // HBSamplevalues=HBData; //Data from Palm.
   //cout <<"\n"<<HBSamplevalues;
   //Calculate the Rational RF function at the sample points. m bar Rational functions for m bar Sample points

   //   cout <<"\nA Sample Values:"<<HASamplevalues;
   //cout <<"\nB Sample Values:"<<HBSamplevalues;

  //Calculate the Rational RF function at the sample points. m bar Rational functions for m bar Sample points
   RF.SetLength(mbar);
   for(int ii=0;ii<mbar;ii++)
     {
         RF[ii]=HASamplevalues[ii]/HBSamplevalues[ii];
     }

   //timevar3=clock();
   vec_ZZ_p HBData;
   HBData.SetLength(bsize); //This is a redundant albatross. Just to minimise perturbation in palm program

   long d = calc_diff_ma_mb(HAData,HBData); //Calculate d =|A|-|B|

  bvector.SetLength(mbar); //This vector holds the RHS of the Vandermonde Equations (As in MATRIX FORM)

   bvector=construct_RHS_constant_vector(RF,samplepoints,mbar,mbar,d);  //Computing bvector. Note mbar,mbar argument since rank=mbar so far

   vmatrix = construct_vandermonde_matrix(samplepoints,RF,bvector,mbar,d);//Constructing the Vandemonde Matrix
   //cout <<"\nVandermonde matrix including RHS Augmented matrix :\n"<<vmatrix;
   //cout <<vmatrix;
   //timevar4=clock();
   long mabar=calc_mabar(mbar,d);
   long mbbar=calc_mbbar(mbar,d);;
   long rank=gauss(vmatrix,mbar); //GAUSSIAN ELIMINATION
   if (rank > mabar+mbbar) rank = mabar+mbbar; //TRY
   //timevar5=clock();

   bvector.SetLength(rank); //Take only the rank number of bvector constituents

   //Making the Reduced VanderMonde Matrix rank*rank
   mat_ZZ_p vredmatrix;// Vandermonde Reduced Matrix using only rank*rank dimensions
   vredmatrix.SetDims(rank,rank);
   vredmatrix = makeA(vmatrix,rank,mbar); //Make the VanderMonde MAtrix with size Rank*Rank
   //cout <<"\n A = \n"<<vredmatrix;


   //Making the RHS of the Vandermonde Equations
   bredvector.SetLength(rank);
   bredvector = makeB(vmatrix,rank,mbar,d);
   //cout <<"\nB = \n"<<bredvector;


   //Now Solving the Vandemonde Matrix by Back Substitution
   vec_ZZ_p coeff_vector; //Holds the solution

   //long mbbar=calc_mbbar(mbar,d);
   coeff_vector.SetLength(mabar+mbbar);
   //coeff_vector.SetLength(rank);
   //cout << "\nRANK:"<<rank;
   //cout << "\nTC:"<<mabar+mbbar;
   coeff_vector=backsubstitutesolve(vredmatrix,bredvector,rank,mabar+mbbar);
   //cout <<"\nSolved Coefficients :\n"<<coeff_vector;
   //timevar6=clock();

   //long mabar = calc_mabar(mbar,d); //Calculating mabar

   vec_ZZ_p coeffP;
   vec_ZZ_p coeffQ;

   if(rank >mabar)
     {
       coeffP.SetLength(mabar+1); //+1 is added beacuse the polynomial is monic and we need to add this fact as the highest degree term p0
       coeffQ.SetLength(1+mabar+mbbar-mabar); //Adding 1 for q0
     }
   else
     {
       coeffP.SetLength(mabar+1);
       coeffQ.SetLength(1+mbbar);
     }

   long Plength=coeffP.length(); //Sizes of P & Q
   long Qlength=coeffQ.length();

   coeffP[Plength-1]=1; //Setting the coeff corresponding to the highest power = 1 (Monic)
   coeffQ[Qlength-1]=1;

   for(ii=0;ii<Plength-1;ii++) //Getting the other coefficients from the Solution to the VanderMonde Equations. p1 = index 0, p2 = index 1 etc.
     coeffP[ii]=coeff_vector[Plength-2-ii]; //Plengh was added a +1 becasue of p0 not being included.
   for(long jj=0;jj<Qlength-1;jj++)
     coeffQ[jj]=coeff_vector[mabar+mbbar-jj-1];


   ZZ_pX P; //These are the ZZ_pX avatars of coeffP & CoeffQ ( which in turn are of type vec_ZZ_p )
   ZZ_pX Q;
   conv(P,coeffP);//Convert coefficients to polynomials
   conv(Q,coeffQ);

   //  cout<<"\nPolynomial P Coefficients :\n"<<P; //Print out the ZZ_pX s
   //cout<<"\nPolynomial Q Coefficients :\n"<<Q;

   ZZ_pX gcdvalue;
   gcdvalue= GCD(P,Q); //GCD calculation

   P=P/gcdvalue; //Reducing by getting rid of common factors
   Q=Q/gcdvalue;

   vec_ZZ_p numerator;
   vec_ZZ_p denominator;

   int returnvalue=0;
   if (match_value(P,Q,PCEVALS,EXTRASPS,EXTRAEVALS)==0) ;
   else
   {
    // cout <"\n B I N G O !!! Match!!! \n";
     numerator.SetLength(Plength-deg(gcdvalue));
     denominator.SetLength(Qlength-deg(gcdvalue));
     //timevar7=clock();
     FindRoots(numerator,P); //Factorize P - Numerator
     //timevar8=clock();
     FindRoots(denominator,Q); //Factorize Q - Denominator
     //timevar9=clock();

//     cout<<"PC Data not in PALM: "<<numerator;
//     cout<<"\n\nPALM data not in PC: "<<denominator;
//     cout <<"\n";
     returnvalue=1;
     //for (long ii=0; ii<numerator.length();ii++) //USE IF UPLOAD IS DESIRED
     //cout<<"\n"<<numerator[ii]; //IN THAT CASE COMMENT OUT STATS

/* 
   The following portion of code was created by Avi in order to convert Sachin's output
   into standard integer arrays.
*/

	   strstream converter1, converter2;
	   char buffer[20];
	   int *num, *den;

	   num = new int[numerator.length()+1];		//num array holds PC data not in PALM
	   num[0] = numerator.length();				//first element is the dynamic array length
	   if( numerator.length() > 0 )				//convert the vec_ZZ to array of ints (num)
	   {
		   converter1 << numerator;				//output in ASCII form to stringstream
		   for( int x = 1; x < numerator.length()+1; x++ )
		   {
			   converter1 >> buffer;			//read the ASCII data back and convert it to ints
			   if( x == 1)						//have to watch out for initial '[' in ASCII output
				   num[x] = atoi(buffer+1);
			   else
				   num[x] = atoi(buffer);
		   }
	   }

	   den = new int[denominator.length()+1];	//perform same conversion for den array
	   den[0] = denominator.length();			//this array will hold PALM data not in PC
	   if( denominator.length() > 0 )
	   {
		   converter2 << denominator;
		   for( int x = 1; x < denominator.length()+1; x++ )
		   {
			   converter2 >> buffer;
			   if( x == 1)
				   den[x] = atoi(buffer+1);
			   else
				   den[x] = atoi(buffer);
		   }
	   }

	   *missingdata = new int*[2];				//create an array of two int pointers...
	   (*missingdata)[0] = num;					//...and initialize them to num and den arrays
	   (*missingdata)[1] = den;					//...do this so that you can return only one var.

   }
   return returnvalue;
}






/* FUNCTION: runProb
 * PURPOSE: Given pointers to arrays of the local machine's hash  *
 * values, the remote machines hash values, and the number of     *
 * records present on the remote machine, this function prepares  *
 * the arguments for and runs the ProbSync algorithm, as many     *
 * times as required/able to reconcile the two databases. The     *
 * returned result is explained in further detail in the function *
 * header of the reconcileProb function header.                   *
 * DATE: 5/22/02                                                  *
 * CONSTANTS: 32749 - size of the finite field (in 2 places!)     *
 *            3 - number of sample points for confirmation        *
 *                that the result is accurate (all over)          *
 *            4 - number of rounds to execute                     *
 *			  DIFFERENCES                                         *
 * MODULES: ProbSync                                              */
int **runProb( int *pchash, int **hhpolyptr, int count, BYTE db )
{
	int **missingdata = NULL;
	int *temppoly = NULL;
	int temp, numrecs, index, recindex = 1;
	CRawRecordInfo rawrecord;
	int *hhpoly = *hhpolyptr;


	vec_ZZ_p A;
	vec_ZZ_p B;

	vec_ZZ_p EXTRASPS;		//These are random sample points used in checking the Rational Function
	vec_ZZ_p EXTRAEVALS;	//These are evaluations of the characteristic polynomial at the EXTRASPS
  
	long d;
	ZZ_p::init(to_ZZ("32749")); //Prime field.

	int ctr = 0;

	d = count;

	A.SetLength(pchash[0]);					//set length of pc-hash vector
	for( int x = 0; x < pchash[0]; x++ )	//then read data into it
		A[x] = to_ZZ_p(pchash[x+1]);

	B.SetLength(hhpoly[0]-3);				//set length of handheld char. poly vector
	for( x = 0; x < hhpoly[0]-3; x++ )		//then read data into it
		B[x] = to_ZZ_p(hhpoly[x+1]);

	EXTRASPS.SetLength(3);					//set the extra sample points
	EXTRASPS[0] = -1*(hhpoly[0]/2 - 1);
	EXTRASPS[1] = (hhpoly[0]/2 - 1);
	EXTRASPS[2] = -1*(hhpoly[0]/2);

	EXTRAEVALS.SetLength(3);		//get the evaluations at the three points...
	for( x = 0; x < 3; x++ )		//...for the computer to compare against.
		EXTRAEVALS[x] = to_ZZ_p(hhpoly[x+hhpoly[0]-2]);

	//WHILE A corresponds to the data being held in the PC, B is the set holding the characteristic polynomial evaluations  of the PALM. mm = mbar.
	while(reconcileProb(hhpoly[0]-3,d,A,B,EXTRASPS,EXTRAEVALS,to_ZZ("32749"), &missingdata)==0)//THIS RECONCILE HAS DIFFERENT PARAMETERS THAN THE ONE USED PREVIOUSLY FOR JUST RECONCILIATION SIMULATION AND TESTING.
	{
		//if this portion of the code is executed, it means that the algorithm needs more
		//characteristic evaluations to complete (ie. execute another round)
		
		//We must download more sample points from the palm, but we can still make use of our existing
		//set of sample points. We will save the existing ones first, then allocate more space and 
		//download the new ones and add in the old ones.

		temppoly = new int[hhpoly[0]+1];				//save the old sample points in a temporary location	
		for( int x = 0; x < hhpoly[0]+1; x++ )
			temppoly[x] = hhpoly[x];

		delete(hhpoly);									//free up the old sample points space

		memset(&rawrecord, 0, sizeof(CRawRecordInfo));  //read the new sample points from the remote machine
		rawrecord.m_pBytes = (BYTE*)new short[DIFFERENCES];
		rawrecord.m_FileHandle = db;					//in this case, simply download (up to a max size) all the
		rawrecord.m_RecIndex = ++recindex;				//sample points in the next highest PalmPilot record
		rawrecord.m_TotalBytes = DIFFERENCES * sizeof(short);
		SyncReadRecordByIndex(rawrecord);

		numrecs = (rawrecord.m_pBytes[0] << 8) | rawrecord.m_pBytes[1];		//see how many new sample points were downloaded

		hhpoly = new int[temppoly[0]+numrecs+1];	//allocate new (larger) space that will fit both the existing
		hhpoly[0] = temppoly[0]+numrecs;			//and the new sample points. Set its first index to its size.

		for( x = 0; x < temppoly[0]; x++ )  		//copy the old sample points back into the new array
			hhpoly[x+1] = temppoly[x+1];

		for( index = 2; x < numrecs + temppoly[0]; index += 2, x++ )	//copy the new sample points to the new array too
		{
			temp = (rawrecord.m_pBytes[index] << 8) | rawrecord.m_pBytes[index+1];	//need conversion from Big-Endian
			hhpoly[x+1] = temp;
		}

		delete( rawrecord.m_pBytes );

		ctr++;						//keep track of how many rounds we have executed
		if( ctr > 4 )				//if we have to go more than 4 times, just quit
		{	
			missingdata = NULL;
			return( missingdata );
		}

		//from this point on, we have the data we need to run reconcileProb again
		//we just have to format the new args correctly.
		B.SetLength(hhpoly[0]-3);			//set length of handheld char. poly vector
		for( x = 0; x < hhpoly[0]-3; x++ )	//then read data into it
			B[x] = to_ZZ_p(hhpoly[x+1]);

		EXTRASPS.SetLength(3);				//set the values of the extra sample points
		EXTRASPS[0] = -1*(hhpoly[0]/2 - 1);
		EXTRASPS[1] = (hhpoly[0]/2 - 1);
		EXTRASPS[2] = -1*(hhpoly[0]/2);


		EXTRAEVALS.SetLength(3);			//set the actual evaluations at the values
		for( x = 0; x < 3; x++ )
			EXTRAEVALS[x] = to_ZZ_p(hhpoly[x+hhpoly[0]-2]);

		delete(temppoly);
	  }

	*hhpolyptr = hhpoly;	//since we have allocated and deallocated the remote machine's sample points we have to make
	return( missingdata );	//sure that the function arguments still point to the right location.
} 
