#include "\Common\Constants.h"
#include "\Palm\PalmCond\PalmCond.h"
#include "\Palm\PalmMemoDb\PalmMemoDb.h"
#include "\Palm\PalmHashDb\PalmHashDb.h"

/* FUNCTION: isNotIn
 * PURPOSE: Given an integer value, a pointer to an array of integers, and the *
 * size of the array pointed to, this function will scan through the array and *
 * return a 0 if the integer value appears at least once in the array and a 1  *
 * if it does not appear in the array.                                         *
 * DATE: 5/23/02                                                               *
 * CONSTANTS: none                                                             *
 * MODULES: PalmCond                                                           */
UInt16 isNotIn( UInt16 lookfor, UInt16 *lookin, UInt16 size )
{
	UInt16 x;
	for( x = 0; x < size; x++ )	
		if( lookfor == lookin[x] )
			return(0);
	return(1);
}


/* FUNCTION: palmRPC
 * PURPOSE: This function contains the commands for the Palm's RPC as called from the PC during a hotsync. In the RPC, the memo  *
 * text corresponding to the hash values that the remote machine is missing are found and placed in the response buffer. After   *
 * the response is given (the PC is then free to continue its processing), the Palm incorporates the memo text that it recieved  *
 * from the remote machine corresponding to the hash values that the Palm was itself missing. This work is done in parallel with *
 * the remote machine, but typically takes longer, so the time of the synchronization is dependant on this process.              *
 * DATE: 6/20/02                                                                                                                 *
 * CONSTANTS: REPLYDATASIZE - the largest size of the reply data array                                                           *
 * MODULES: PalmCond                                                                                                             *
 * What is the purpose of the three lines, starting with "mytempstorage = MemHandleNew( size );"?                                */
void palmRPC( MemPtr cmdPBP )
{
	DlkCallAppReplyParamType theReplyParams;
	SysAppLaunchCmdHandleSyncCallAppType* theCommandPtr;
	char *indexes, *data, *returnmemos, *tempstorage;
	UInt16 size, numdeletedmemosplaced, deleted[50], deletedindex = 0;
	DmOpenRef docDB = NULL, idocDB = NULL, hashDB = NULL, polyDB = NULL;
	MemHandle myArray, mytempstorage;

			// Cast the cmdPBP to a SysAppLaunchCmdHandleSyncCallAppType 
			// pointer so that we can work with it.
			theCommandPtr = (SysAppLaunchCmdHandleSyncCallAppType*)cmdPBP;

			// Do whatever work is necessary here.  If you set the m_wActionCode
			// field in your CCallModuleParams class on the desktop, then you 
			// can handle that code by looking at the action field of theCommandPtr
			// (i.e.) if (theCommandPtr->action == 1)
			data = theCommandPtr ->paramP;
			size = theCommandPtr ->dwParamSize;

			openDatabases( &docDB, &idocDB, &hashDB, &polyDB );	//open the databases using local variables (globals not allowed)

			// Create the reply to send back to the desktop:
			// First clear out all the fields.  This is necessary so that the reserved
			// fields are set to NULL.
			MemSet( &theReplyParams, sizeof(DlkCallAppReplyParamType), 0 );
	
			// Set the size of the reply.  This is required.
			theReplyParams.pbSize = sizeof(DlkCallAppReplyParamType);
	
			// Set the result code.  Normally this will be set to zero unless you want
			// to send an error code back to the desktop.
			theReplyParams.dwResultCode = 0;
	
			myArray = MemHandleNew( REPLYDATASIZE );	//use a dynamically allocated array to store our reply data
			indexes = MemHandleLock( myArray );
			
			//this is the heart of the RPC, it puts any deleted memos in the reply data, and provides the text for the remote machine's missing memos
			numdeletedmemosplaced = placeDeletedMemos( deleted, indexes, docDB, hashDB, polyDB );		
			data = placePCRequestedMemos( data, indexes, &returnmemos, deleted, numdeletedmemosplaced, idocDB, hashDB );
			updateDocumentCount( docDB, polyDB );

			mytempstorage = MemHandleNew( size );
			tempstorage = MemHandleLock( mytempstorage );
			MemMove( tempstorage, data, size );
			
			theReplyParams.resultP = indexes;
			theReplyParams.dwResultSize = returnmemos - indexes;	
			
			// Fill in the DL reference pointer.  This is required.
			theReplyParams.dlRefP = theCommandPtr->dlRefP;
	
			// Set the handled field to true.  This is required to let the desktop
			// know that the sysAppLaunchCmdHandleSyncCallApp was handled.  If you
			// don't set this to true, the call to SyncCallRemoteModule will return
			// SYNCERR_UNKNOWN_REQUEST.
			theCommandPtr->handled = true;
			
			// Finally, set the replyErr field by passing the reply parameters to
			// DlkControl.  This is required for the DLServer to properly handle the
			// reply request.
			theCommandPtr->replyErr = DlkControl(dlkCtlSendCallAppReply, &theReplyParams, NULL);	//after this call, remote machine is unblocked

			//do the time consuming process (saving the data, updating the meta-data) in parallel with the PC
			data = saveDataOntoPalm( tempstorage, docDB, idocDB, hashDB, polyDB );
			MemHandleUnlock( mytempstorage);
			MemHandleFree( mytempstorage );
			MemHandleUnlock( myArray );
			MemHandleFree( myArray );

			closeDatabases( docDB, idocDB, hashDB, polyDB );
}


/* FUNCTION: placeDeletedMemos
 * PURPOSE: Given a pointer to the data that will be sent back to the remote computer, and references to the pre-opened   *
 * data and meta-data databases, this function will place (in the data to be sent back) the index/hash value pairs of     *
 * each memo that is marked for deletion on the PalmPilot for the PC to delete it remotely. (The index refers to the      *
 * index into the document database.) The function returns the number of deleted documents.                               *
 * DATE: 6/21/02                                                                                                          *
 * CONSTANTS: none                                                                                                        * 
 * MODULES: PalmCond, PalmHashDb                                                                                          *
 * SUGGESTION: The comments are not completely accurate yet, because they do not describe the purpose of the "deleted" arg*
 * THIS FUNCTION DOES NOT WORK - THE INSIDE OF THE IF STATEMENT WILL NOT WORK!!! DELETED MEMOS ARE NOT SUPPORTED YET!     */
int placeDeletedMemos( UInt16 *deleted, char *indexes, DmOpenRef docDB, DmOpenRef hashDB, DmOpenRef polyDB )
{
	MemHandle myRecord;
	UInt16 numdeleted, dummy, index, deletedindex = 0, y=0;
	HashRecordType *hashrecord;

	numdeleted = numTotalMemos(docDB) - numNonDeletedMemos(docDB); 	//get the number of records marked for deletion
	if( numdeleted != 0 )								//if there are records to be deleted
	{
		myRecord = DmQueryRecord( hashDB, 0 );
		hashrecord = MemHandleLock( myRecord );
		for( y = 0, index = numTotalMemos(docDB)-1; y < numdeleted; y++, index-- )		//for each deleted record
		{												
			writeInt16( &(indexes[4*y+2]), index);										//write its index in the document database	
			dummy = findRecordWithDocIndex(index, hashDB );								//(so that it will be sent later to the PC for remote deletion)
			deleted[deletedindex++] = hashrecord[dummy].hashval;						//get the hashvalue of the deleted document
			updatePolynomialDb( hashrecord[dummy].hashval, polyDB, docDB, 1 );
			writeInt16( &(indexes[4*y+4]) , hashrecord[dummy].hashval );				//write its hashvalue (so that the PC can check it)
			deleteHashIndex( dummy, hashDB );                                           //we don't need the hashvalue entry anymore, so delete it.
		}
		MemHandleUnlock( myRecord );
	}
	writeInt16( indexes, numdeleted );				//provide a count of how many index/hash pairs we gave
	return(y);
}


/* FUNCTION: placePCRequestedMemos
 * PURPOSE: Given the pointer to the data sent from the remote machine, the data to be sent back to the remote machine, a pointer to   *
 * a list of hash values of deleted memos and a count of the number of deleted memos, this function reads the remote machine's missing *
 * hash values and places the text of the memos corresponding to those hashes into the data to be sent back to the remote machine.     *
 * Checking is also done to ensure that memos marked for deletion are not sent back. The return value is a pointer to a point just     *
 * beyond the data written by this function, so that any other functions that wish to send data to the remote machine won't interfere. *
 * DATE: 6/21/02                                                                                                                       *
 * CONSTANTS: 50 - denotes the maximum possible size of the list of deleted hashvalues.                                                *
 * MODULES: PalmCond                                                                                                                   */
char *placePCRequestedMemos( char *data, char *indexes, char **returnmemos, UInt16 *deleted, UInt16 numdeleted, DmOpenRef idocDB, DmOpenRef hashDB )
{
	UInt16 numhashes, hash, index, x;
	char *readdata, buffer[MEMOSIZE];

	readdata = data;
	numhashes = readInt16( readdata );			//get the memo texts that correspond to the remote machine's missing hashes
	readdata += 2;
	*returnmemos = &(indexes[4*numdeleted+2]);	//jump to the right location in the returned data (we don't want to overwrite other data to be returned)
	for( x = 0; x < numhashes; x++ )		//for each of the remote machine's missing hash values
	{										
		hash = readInt16( readdata );		//get the hash value
		if( isNotIn( hash, deleted, 50 ) )	//check to see that it is not a memo that is marked for deletion
		{
			index = findIndexWithHashVal( readInt16(readdata), hashDB, NULL );	//get the index in the document database corresponding to the hash
			getMemoText( idocDB, index, buffer );
			StrCopy( *returnmemos, buffer );
		}
		else
			*returnmemos[0] = '\0';			//if it is marked for deletion, just put in a null terminator
				
		*returnmemos += StrLen( *returnmemos ) + 1;
		readdata += 2;	
	}
	return(readdata);
}


/* FUNCTION: readInt16                                                            *
 * PURPOSE: Given a pointer to the first byte of a two byte number in Big-Endian  *
 * format, this function will return that same number in Little-Endian.           *
 * DATE: 5/23/02                                                                  *
 * CONSTANTS: none                                                                *
 * MODULES: PalmCond                                                              */
UInt16 readInt16( char *readfrom )
{
	char buffer[2];
	UInt16 *value;
	
	buffer[0] = readfrom[0];
	buffer[1] = readfrom[1];
	
	value = (UInt16*)&buffer;
	return(*value);
}


/* FUNCTION: saveDataOntoPalm
 * PURPOSE: Given the missing memo texts that were given to us by the remote machine after it     *
 * ran the synchronization algorithm, and references to the data and meta-data databases, this    *
 * function will save all the memos in a "block" fashion (all at once) rather than peicemeal.     *                
 * DATE: 5/23/02                                                                                  *
 * CONSTANTS: 1 - Indicates to saveTestString that only the document databases are to be updated  *
 *                not the other meta-data.                                                        *
 * MODULES: PalmCond                                                                              */
char *saveDataOntoPalm( char *data, DmOpenRef docDB, DmOpenRef idocDB, DmOpenRef hashDB, DmOpenRef polyDB )
{
	UInt16 nummemos,x;
	char *readdata = data;
	MemHandle hashbuffer;
	
	HashRecordType *hashvalues;
	
	nummemos = readInt16(data);		//get the number of memos that the PC transferred to us
	
	if( nummemos != 0 )			    //only execute if there are memos that need to be written to the PalmPilot
	{
		hashbuffer = MemHandleNew( nummemos * sizeof(HashRecordType) );	//create a buffer that will hold the memos
		hashvalues = MemHandleLock( hashbuffer );
		
		readdata += 2;
		for( x = 0; x < nummemos; x++ )	            //read the memos and save them to the document (data) databases
		{
			hashvalues[x].hashval = readInt16(readdata + StrLen(readdata) + 1);		//read the memo hashvalue
			hashvalues[x].docindex = saveTestString(readdata,docDB,idocDB);         //save the memo text
			readdata += StrLen(readdata)+3;
		}
				
		blockInsertHashIndexes( nummemos, hashvalues, hashDB );              //now, update the hash database with all the new indexes 
		
		blockUpdatePolynomialDb( nummemos, hashvalues, polyDB, docDB, 0 );   //also, update the polynomial database to reflect the new additions
		
		MemHandleUnlock( hashbuffer );				//get rid of the temporary buffer
		MemHandleFree( hashbuffer );
	}
	return(readdata);
}


/* FUNCTION: WriteInt16                                                           *
 * PURPOSE: Given a pointer to the first byte of a two byte storage location, and *
 * a 16bit integer, this function will place that integer in Big-Endian format in *
 * the storage location.                                                          *
 * DATE: 5/23/02                                                                  *
 * CONSTANTS: none                                                                *
 * MODULES: PalmCond                                                              */
void writeInt16( char *writeto, UInt16 value )
{
	writeto[1] = *((char*)&value);
	writeto[0] = *((char*)&value + 1);
}
