#include <stdio.h>
#include <stdlib.h>
#include "common\constants.h"
#include "PC\PCGenMemo\PCGenMemo.h"
#include "PC\PCMemoFile\PCMemoFile.h"

/* FUNCTION: main                                                                        *
 * PURPOSE: This application will modify the memos stored on the PC through command line *
 * parameters. If the first parameter (the action code, argv[1]) is zero, then the app   *
 * takes three further arguments: the number of memos to generate, the size of each      *
 * generated memo and the number of symettric differences desired between the memos      *
 * generated on the PC and the PalmPilot. The program generates the memos according to   *
 * these arguments and then saves them, OVERWRITING any memos that were previously saved!*
 * If the action code is one, the app uses the next parameter as memo text to append to  *
 * the existing PC memo file. This app was developed for users automated testing. A      *
 * better UI is available in the PCMemo application.                                     *
 * DATE: 6/14/02                                                                         *
 * CONSTANTS: NUMMEMOS, MEMOSIZE                                                         *
 * MODULES: none                                                                         */
void main( int argc, char *argv[] )
{
	int target, nummemos=0, memosize, symdiff;			//command line args
	memo memodata[NUMMEMOS];					//memo list
	char buffer[MEMOSIZE];						
	char syncmemofilename[300] = "";
	char randomdatafilename[300] = "";
	FILE *randomdata;							//random data file

	strcat( syncmemofilename, PALMPATH );
	strcat( syncmemofilename, "\\syncMemo\\SyncMemo.dat" );
	strcat( randomdatafilename, ROOTPATH );
	strcat( randomdatafilename, "\\testto~1\\pcmemo\\random.dat" );

	if( argc != 3 && argc != 5 )				//if wrong # of args, print usage and exit
	{
		printf("USAGE: ConsoleMemo n [MemoText] OR [A B C]\n");
		printf("n = action code [0 - create random memos overwriting existing, 1 - append single memo text]\n");
		printf("A = Number of memos to produce\n");
		printf("B = Size (in characters) of each memo\n");
		printf("C = # of symmetric differences between the PC and PalmPilot\n");
		return;
	}

	initializeNewData(memodata);

	if( atoi(argv[1]) )		//if the action code = 1, we are appending to the existing file
	{
		target = loadFile( memodata, syncmemofilename );
		memodata[target].hash = calcHashValue(argv[2]);		//add our new memo to the end
		memodata[target].data = argv[2];
		target++;
	}
	else
	{
		target = atoi(argv[2]);					//convert command line args to ints
		memosize = atoi(argv[3]);
		symdiff = atoi(argv[4]);
		printf("# memos = %d\tMemo Size = %d\tSymmetric Differences = %d\n", nummemos, memosize, symdiff);

		randomdata = fopen( randomdatafilename, "r");		//initialize the random data file

		for( int x = 0; x < target; x++ )		//for each memo that we want to make
		{
			strcpy( buffer, makeMemo( x, memosize, randomdata) );	//copy the memo returned by the makeMemo function
				
			//only proceed if this memo is actually going to be used (that is, if it does not fall into the symmettric 
			//difference area that we don't want to generate.
			if(!( (x >= target - symdiff) && ( x < target - symdiff + symdiff/2 )))
			{
				memodata[nummemos].data = new char[strlen(buffer)+1];		//fill in an index in the memo array
				strcpy( memodata[nummemos].data, buffer );
				memodata[nummemos].hash = calcHashValue( memodata[nummemos].data );
				nummemos++;
			}
		}
		fclose(randomdata);
	}

	saveFile(syncmemofilename, nummemos, memodata );
	return;
}