// PCMemoView.cpp : implementation of the CPCMemoView class
//

#include "stdafx.h"
#include "PCMemo.h"

#include "PCMemoDoc.h"
#include "PCMemoView.h"

#include "Memodialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define HASHDEBUG

/////////////////////////////////////////////////////////////////////////////
// CPCMemoView

IMPLEMENT_DYNCREATE(CPCMemoView, CFormView)

BEGIN_MESSAGE_MAP(CPCMemoView, CFormView)
	//{{AFX_MSG_MAP(CPCMemoView)
	ON_BN_CLICKED(IDC_SAVE, OnSave)
	ON_BN_CLICKED(IDC_STOP, OnStop)
	ON_LBN_SELCHANGE(IDC_LISTMEMOS, OnSelchangeListmemos)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	ON_BN_CLICKED(IDC_SAVEFILE, OnSavefile)
	ON_BN_CLICKED(IDC_LOADFILE, OnLoadfile)
	ON_BN_CLICKED(IDC_FILLINDATA, OnFillindata)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CFormView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPCMemoView construction/destruction

CPCMemoView::CPCMemoView()
	: CFormView(CPCMemoView::IDD)
{
	//{{AFX_DATA_INIT(CPCMemoView)
	m_memostring = _T("");
	//}}AFX_DATA_INIT
	// TODO: add construction code here
	
	initData();
}

CPCMemoView::~CPCMemoView()
{
	freeData(memodata);
}

void CPCMemoView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPCMemoView)
	DDX_Control(pDX, IDC_LISTMEMOS, m_ListTitles);
	DDX_Text(pDX, IDC_MEMODATA, m_memostring);
	//}}AFX_DATA_MAP
}

BOOL CPCMemoView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

void CPCMemoView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();

}


/////////////////////////////////////////////////////////////////////////////
// CPCMemoView diagnostics

#ifdef _DEBUG
void CPCMemoView::AssertValid() const
{
	CFormView::AssertValid();
}

void CPCMemoView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CPCMemoDoc* CPCMemoView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CPCMemoDoc)));
	return (CPCMemoDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CPCMemoView message handlers


/* FUNCTION: initializeNewData
 * PURPOSE: This function will initialize the memo application to hold new data. This *
 * includes resetting all the information in the memo array.                          *
 * DATE: 6/21/02                                                                      *
 * CONSTANTS: NUMMEMOS                                                                *
 * MODULES: PCGenMemoApp                                                              */
void CPCMemoView::initData()
{
	nummemos = 0;								//reset the number of memos loaded from file
	currentmemo = -1;							//reset the current selected memo

	initializeNewData( memodata );				//reset all the info in the memo array
}


/* FUNCTION: saveMemo                                                                *
 * PURPOSE: This function will save the current memo (specified by the "currentmemo" *
 * global variable)                                                                  *
 * DATE: 6/21/02                                                                     *
 * CONSTANTS: none                                                                   *
 * MODULES: PCGenMemoApp                                                             */
void CPCMemoView::saveMemo()
{
	if( currentmemo != -1 )					//if the memo is not a new memo
		delete(memodata[currentmemo].data);		//then deallocate what is already in the array at the save location
	else									//if it is a new memo, set the current memo value
		currentmemo = nummemos;					//and set the current memo to the last memo index

	UpdateData();
	createMemo( (LPCTSTR)m_memostring, calcHashValue( (LPCTSTR)m_memostring ), &nummemos, memodata );
	addItemToTitles( currentmemo );
	newMemo();								//now that the memo has been saved... set up a fresh new memo
}


/* FUNCTION: newMemo                                                             *
 * PUROSE: This function will effectively "de-select" any current memo and allow *
 * the user to enter a new memo.                                                 *
 * DATE: 5/29/02                                                                 *
 * CONSTANTS: none                                                               *
 * MODULES: PCGenMemoApp                                                         */
void CPCMemoView::newMemo()
{
	currentmemo = -1;
	m_memostring = "";
	UpdateData(false);
}


/* FUNCTION: getMemo                                                               *
 * PURPOSE: Given an index into the memo array, this function will display that    *
 * memo in the Edit box of the application and set that memo to be the current one *
 * DATE: 5/29/02                                                                   *
 * CONSTANTS: none                                                                 *
 * MODULES: PCGenMemoApp                                                           */
void CPCMemoView::getMemo(int index)
{
	currentmemo = index;
	m_memostring = getMemoText(memodata, currentmemo);	//display the memo in the Edit box
	UpdateData(false);									//refresh the window display
}


/* FUNCTION: deleteMemo                                                                  *
 * PURPOSE: This function deletes from the memo array the memo at the index specified by *
 * the global variable "currentmemo". This usually means the memo that is currently being*
 * displayed in the edit window. It bumps down all the memos that are past the memo to be*
 * deleted so that the array maintains continuity and also removes the memo's title from *
 * the Title List.                                                                       *
 * DATE: 5/29/02                                                                         *
 * CONSTANTS: none                                                                       *
 * MODULES: PCGenMemoApp, PCGenMemo                                                      */
void CPCMemoView::deleteMemo()
{
	if( currentmemo != -1 )										//check to make sure that a memo is even selected
	{
		for( int x = currentmemo; x < nummemos-1; x++ )			//for each memo AFTER the one to be deleted, move it
			copyMemoEntry( &(memodata[x+1]), &(memodata[x]) );	//onto the memo that is one index below it.
		nummemos--;												//keep track of the number of memos
		removeItemFromTitles( currentmemo );					
		currentmemo = -1;										//no new memo should be selected
	}

	m_memostring = "";											//no new memo data should be displayed
	UpdateData(false);											//refresh the window to reflect the changes
}





/* FUNCTION: OnSave                                                               *
 * PURPOSE: This function is called when the user presses the "Save Memo" button. *
 * DATE: 5/29/02                                                                  *
 * CONSTANTS: none                                                                *
 * MODULES: PCGenMemoApp                                                          */
void CPCMemoView::OnSave() 
{
	saveMemo();
}


/* FUNCTION: OnStop                                                              *
 * PURPOSE: This is a dummy function provided to allow a developer to stick in a *
 * breakpoint at the "for" statement, which will be called when the user presses *
 * the "STOP" button.                                                            *
 * DATE: 5/29/02                                                                 *
 * CONSTANTS: none                                                               *
 * MODULES: PCGenMemoApp                                                         */
void CPCMemoView::OnStop() 
{
	for( int x = 0; x < 1; x++ );
}


/* FUNCTION: addItemToTitles                                                          *
 * PURPOSE: Given the index in the memo array, this function will add up to the first *
 * forty characters of the memo string to the Title List at the same index, and will  *
 * delete any memo currently occupying that index in the Title List.                  *
 * DATE: 6/25/02                                                                      *
 * CONSTANTS: 40 - maximum length of the title                                        *
 * MODULES: PCGenMemoApp                                                              */
void CPCMemoView::addItemToTitles(int index)
{
	CString title = "";
	CString buffer( getMemoText( memodata, index ) );
//	CString buffer = memodata[index].data;

	if( buffer.GetLength() > 40 )					//if the memo string is larger than 40 chars
		title.Format("%s...", buffer.Left(40) );	//then only put the first 40 followed by a "..."
	else
		title.Format("%s", buffer.Left(40) );		//otherwise just put the memo itself without a trailing "..."

	if( m_ListTitles.GetTextLen(index) != LB_ERR )	//check to see if there already exists a string at the specified index
		m_ListTitles.DeleteString( index );			//if there is a string there, then remove it from the list
	
	m_ListTitles.InsertString( index, title );
}


/* FUNCTION: OnSelchangeListmemos                                                    *
 * PURPOSE: This function is called when the user clicks a memo title other than the *
 * one currently highlighted in the Memo Titles list. Since the memo title that is   *
 * highlighted corresponds to the memo being edited, then this will result in a      *
 * change of the memo being edited. NO SAVE WILL OCCUR FOR THE CURRENT MEMO BEING    *
 * EDITED!                                                                           *
 * DATE: 5/29/02                                                                     *
 * CONSTANTS: none                                                                   *
 * MODULES: PCGenMemoApp                                                             */
void CPCMemoView::OnSelchangeListmemos() 
{
	getMemo(m_ListTitles.GetCurSel());
}


/* FUNCTION: removeItemFromTitles                                                         *
 * PURPOSE: Given an index, this function will remove the title of the memo corresponding *
 * to that index from the Memo Titles list.                                               *
 * DATE: 5/29/02                                                                          *
 * CONSTANTS: none                                                                        *
 * MODULES: PCGenMemoApp                                                                  */ 
void CPCMemoView::removeItemFromTitles(int index)
{
	m_ListTitles.DeleteString( index );
}





/* FUNCTION: OnDelete                                                                        *
 * PURPOSE: This function is called when the user presses the "Delete Memo" button. It calls *
 * the deleteMemo function to delete the appropriate memo.                                   *
 * DATE: 5/29/02                                                                             *
 * CONSTANTS: none                                                                           *
 * MODULES: PCGenMemoApp                                                                     */
void CPCMemoView::OnDelete() 
{
	deleteMemo();
}


/* FUNCTION: OnSavefile                                                           *
 * PURPOSE: This function is called when the user presses the "Save File" button. *
 * DATE: 5/29/02                                                                  *
 * CONSTANTS: none                                                                *
 * MODULES: PCGenMemoApp                                                          */
void CPCMemoView::OnSavefile() 
{
	char filepath[300];

	strcpy( filepath, PALMPATH );
	strcat( filepath, "\\syncMemo\\SyncMemo.dat" );
	saveFile(filepath, nummemos, memodata );
}


/* FUNCTION: OnLoadfile                                                           *
 * PURPOSE: This function is called when the user presses the "Load File" button. *
 * This function will load all the memos from a specific memo file. It will reset *
 * the application before it does this, so that the only information displayed    *
 * after the loading will be the memos and titles that are loaded from the file.  * 
 * DATE: 6/25/02                                                                  *
 * CONSTANTS: location of the memo data file                                      *
 * MODULES: PCGenMemoApp                                                          */
void CPCMemoView::OnLoadfile() 
{
	char syncmemofilename[300] = "";

	strcat( syncmemofilename, PALMPATH );
	strcat( syncmemofilename, "\\syncMemo\\SyncMemo.dat" );

	removeAllTitles();
	
	nummemos = loadFile( memodata, syncmemofilename );
	for( int x = 0; x < nummemos; x++ )				//for each memo loaded, add its title
		addItemToTitles(x);
}


/* FUNCTION: OnFillindata                                                            *
 * PURPOSE: This function is called when the user presses the "Fill In Data" button. *
 * The function controls the popping up of a dialog to get the user to fill in values*
 * for the number, size and symettric differences from the Palm for the memos that   *
 * will be created. Once these memos are created, all other memos that were loaded   *
 * before in the application will be deleted.                                        *
 * DATE: 5/29/02                                                                     *
 * CONSTANTS: 50000 - maximum number of characters provided by the random char file  *
 * MODULES: PCMemoApp, PCGenMemo                                                     */
void CPCMemoView::OnFillindata() 
{
	Memodialog dialog;
	char buffer[MEMOSIZE];

#ifndef JOKESFILE									//decide which file to open for the memo data 
	FILE *randomdata = fopen("random.dat", "r");
#endif
#ifdef JOKESFILE
	FILE *randomdata = fopen("jokes.txt", "r");
#endif

#ifdef HASHDEBUG									//decide if we want to output all the hash values to a file
	FILE *output = fopen("c:\\hashout.tst", "w" );
#endif

#ifdef EXPORT										//decide if we want to output the memos themselves to a file
	FILE *memooutput = fopen("c:\\memoout.txt", "w" );
#endif

	dialog.m_number = 200;							//set some default values in the dialog box
	dialog.m_size = 120;
	dialog.m_differences = 0;

	if( dialog.DoModal() == IDOK )					//pop up the dialog box
		if( dialog.m_number * dialog.m_size >= 50000 )					//check to see that we don't ovverrun the # of 
			AfxMessageBox("Too many characters required!!!", IDOK );	//chars we provide in the file
		else
		{
			removeAllTitles();											//delete all existing memos
			freeData(memodata);	
			initData();
			for( int x = 0; x < dialog.m_number; x++ )					//for each memo that we want to make
			{

				strcpy( buffer, makeMemo( x, dialog.m_size, randomdata) );	//copy the memo returned by the makeMemo function
				
				//only proceed if this memo is actually going to be used (that is, if it does not fall into the symmettric 
				//difference area that we don't want to generate.
				if(!( (x >= dialog.m_number - dialog.m_differences) && ( x < dialog.m_number - dialog.m_differences + dialog.m_differences/2 )))
				{
				
#ifdef EXPORT										//write the memo to a file, if desired
					fputs( buffer, memooutput );
					fprintf(memooutput, "\t\n" );
#endif

					memodata[nummemos].data = new char[strlen(buffer)+1];		//fill in an index in the memo array
					strcpy( memodata[nummemos].data, buffer );
					memodata[nummemos].hash = calcHashValue( memodata[nummemos].data );

#ifdef HASHDEBUG									//write the hash values to a file, if desired
					fprintf(output, "%d\t%d\n", x, memodata[nummemos].hash );
#endif

					addItemToTitles( nummemos );
					nummemos++;
				}
			}
		}

#ifdef HASHDEBUG					//clean up various files
	fflush( output );
	fclose( output );
#endif

#ifdef EXPORT
	fflush( memooutput );
	fclose( memooutput );
#endif

	fclose( randomdata );
}


/* FUNCTION: removeAllTitles                                                  *
 * PURPOSE: This function will erase all the titles from the Memo Title list. *
 * DATE: 5/29/02                                                              *
 * CONSTANTS: none                                                            *
 * MODULES: PCGenMemoApp                                                      */
void CPCMemoView::removeAllTitles()
{
	m_ListTitles.ResetContent();
}



